package jrummikub.control;

import java.util.ArrayList;
import java.util.List;

import jrummikub.model.GameSettings;
import jrummikub.model.GameState;
import jrummikub.model.IRoundState;
import jrummikub.model.RoundState;
import jrummikub.model.Score;
import jrummikub.util.Connection;
import jrummikub.util.Event;
import jrummikub.util.IEvent;
import jrummikub.util.IListener;
import jrummikub.util.IListener1;
import jrummikub.view.IView;

/**
 * Controls a Game, at some point including all Rounds, starts new Rounds
 */
public class GameControl {
	private GameSettings gameSettings;
	private IView view;
	RoundControl roundControl;
	private GameState gameState;
	private List<Connection> connections = new ArrayList<Connection>();
	private Event endOfGameEvent = new Event();

	/**
	 * Constructor
	 * 
	 * @param gameSettings
	 *            the game settings
	 * @param view
	 *            the view
	 */
	public GameControl(GameSettings gameSettings, IView view) {
		this.gameSettings = gameSettings;
		this.view = view;

		gameState = new GameState();
		gameState.setFirstRoundFirstPlayer((int) (Math.random() * gameSettings
				.getPlayerList().size()));

		connections.add(view.getNewRoundEvent().add(new IListener() {
			@Override
			public void handle() {
				startRound();
			}
		}));

		connections.add(view.getNewGameEvent().add(new IListener() {
			@Override
			public void handle() {
				endGame();
			}
		}));

		connections.add(view.getEndProgramEvent().add(new IListener() {
			@Override
			public void handle() {
				endProgram();
			}
		}));
	}

	/**
	 * Is emitted when the user ends the game and start a new one
	 * 
	 * @return the endOfGameEvent
	 */
	public IEvent getEndOfGameEvent() {
		return endOfGameEvent;
	}

	private void endGame() {
		for (Connection c : connections) {
			c.remove();
		}
		endOfGameEvent.emit();
	}

	/**
	 * Game gets started by initializing the first Round
	 */
	public void startGame() {
		startRound();
	}

	private void startRound() {
		if (roundControl != null) {
			return;
		}

		view.showScorePanel(false);

		IRoundState roundState = new RoundState(gameSettings);

		roundState.setActivePlayerNumber(gameState.getFirstRoundFirstPlayer()
				+ gameState.getScores().size());

		roundControl = new RoundControl(roundState, view);
		roundControl.getEndOfRoundEvent().add(new IListener1<Score>() {

			@Override
			public void handle(Score roundScore) {
				endOfRound(roundScore);
			}
		});

		roundControl.getRestartRoundEvent().add(new IListener() {

			@Override
			public void handle() {
				restartRound();
			}
		});

		roundControl.startRound();
	}

	private void restartRound() {
		roundControl = null;
		startRound();
	}

	private void endOfRound(Score roundScore) {
		gameState.getScores().add(roundScore);

		roundControl = null;
		view.enableWinPanel(true);

		view.getScorePanel().setPlayers(gameSettings.getPlayerList());
		view.getScorePanel().setScores(gameState.getScores());
		view.getScorePanel().setAccumulatedScore(
				gameState.getAccumulatedScore());
		view.getScorePanel().update();
		view.showScorePanel(true);
	}

	private void endProgram() {
		System.exit(0);
	}

}
